# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: btrdb.proto
# Protobuf Python Version: 4.25.0
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()




DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(b'\n\x0b\x62trdb.proto\x12\x05v5api\"Q\n\x0fRawValuesParams\x12\x0c\n\x04uuid\x18\x01 \x01(\x0c\x12\r\n\x05start\x18\x02 \x01(\x10\x12\x0b\n\x03\x65nd\x18\x03 \x01(\x10\x12\x14\n\x0cversionMajor\x18\x04 \x01(\x04\"}\n\x11RawValuesResponse\x12\x1b\n\x04stat\x18\x01 \x01(\x0b\x32\r.v5api.Status\x12\x14\n\x0cversionMajor\x18\x02 \x01(\x04\x12\x14\n\x0cversionMinor\x18\x03 \x01(\x04\x12\x1f\n\x06values\x18\x04 \x03(\x0b\x32\x0f.v5api.RawPoint\"m\n\x14\x41rrowRawValuesParams\x12\x0c\n\x04uuid\x18\x01 \x01(\x0c\x12\r\n\x05start\x18\x02 \x01(\x10\x12\x0b\n\x03\x65nd\x18\x03 \x01(\x10\x12\x14\n\x0cversionMajor\x18\x04 \x01(\x04\x12\x15\n\rtemplateBytes\x18\x05 \x01(\x0c\"u\n\x16\x41rrowRawValuesResponse\x12\x1b\n\x04stat\x18\x01 \x01(\x0b\x32\r.v5api.Status\x12\x14\n\x0cversionMajor\x18\x02 \x01(\x04\x12\x14\n\x0cversionMinor\x18\x03 \x01(\x04\x12\x12\n\narrowBytes\x18\x04 \x01(\x0c\"\x85\x01\n\x16\x41rrowMultiValuesParams\x12\x0c\n\x04uuid\x18\x01 \x03(\x0c\x12\x14\n\x0cversionMajor\x18\x02 \x03(\x04\x12\r\n\x05start\x18\x03 \x01(\x10\x12\x0b\n\x03\x65nd\x18\x04 \x01(\x10\x12\x14\n\x0csnapPeriodNs\x18\x05 \x01(\x03\x12\x15\n\rtemplateBytes\x18\x06 \x01(\x0c\"K\n\x18\x41rrowMultiValuesResponse\x12\x1b\n\x04stat\x18\x01 \x01(\x0b\x32\r.v5api.Status\x12\x12\n\narrowBytes\x18\x02 \x01(\x0c\"*\n\x0bRawPointVec\x12\x0c\n\x04time\x18\x01 \x01(\x10\x12\r\n\x05value\x18\x02 \x03(\x01\"j\n\x14\x41lignedWindowsParams\x12\x0c\n\x04uuid\x18\x01 \x01(\x0c\x12\r\n\x05start\x18\x02 \x01(\x10\x12\x0b\n\x03\x65nd\x18\x03 \x01(\x10\x12\x14\n\x0cversionMajor\x18\x04 \x01(\x04\x12\x12\n\npointWidth\x18\x05 \x01(\r\"\x83\x01\n\x16\x41lignedWindowsResponse\x12\x1b\n\x04stat\x18\x01 \x01(\x0b\x32\r.v5api.Status\x12\x14\n\x0cversionMajor\x18\x02 \x01(\x04\x12\x14\n\x0cversionMinor\x18\x03 \x01(\x04\x12 \n\x06values\x18\x04 \x03(\x0b\x32\x10.v5api.StatPoint\"z\n\x1b\x41rrowAlignedWindowsResponse\x12\x1b\n\x04stat\x18\x01 \x01(\x0b\x32\r.v5api.Status\x12\x14\n\x0cversionMajor\x18\x02 \x01(\x04\x12\x14\n\x0cversionMinor\x18\x03 \x01(\x04\x12\x12\n\narrowBytes\x18\x04 \x01(\x0c\"m\n\rWindowsParams\x12\x0c\n\x04uuid\x18\x01 \x01(\x0c\x12\r\n\x05start\x18\x02 \x01(\x10\x12\x0b\n\x03\x65nd\x18\x03 \x01(\x10\x12\x14\n\x0cversionMajor\x18\x04 \x01(\x04\x12\r\n\x05width\x18\x05 \x01(\x04\x12\r\n\x05\x64\x65pth\x18\x06 \x01(\r\"|\n\x0fWindowsResponse\x12\x1b\n\x04stat\x18\x01 \x01(\x0b\x32\r.v5api.Status\x12\x14\n\x0cversionMajor\x18\x02 \x01(\x04\x12\x14\n\x0cversionMinor\x18\x03 \x01(\x04\x12 \n\x06values\x18\x04 \x03(\x0b\x32\x10.v5api.StatPoint\"s\n\x14\x41rrowWindowsResponse\x12\x1b\n\x04stat\x18\x01 \x01(\x0b\x32\r.v5api.Status\x12\x14\n\x0cversionMajor\x18\x02 \x01(\x04\x12\x14\n\x0cversionMinor\x18\x03 \x01(\x04\x12\x12\n\narrowBytes\x18\x04 \x01(\x0c\"h\n\x10StreamInfoParams\x12\x0c\n\x04uuid\x18\x01 \x01(\x0c\x12\x13\n\x0bomitVersion\x18\x02 \x01(\x08\x12\x16\n\x0eomitDescriptor\x18\x03 \x01(\x08\x12\x19\n\x04role\x18\x64 \x01(\x0b\x32\x0b.v5api.Role\"\x8a\x01\n\x12StreamInfoResponse\x12\x1b\n\x04stat\x18\x01 \x01(\x0b\x32\r.v5api.Status\x12\x14\n\x0cversionMajor\x18\x02 \x01(\x04\x12\x14\n\x0cversionMinor\x18\x03 \x01(\x04\x12+\n\ndescriptor\x18\x04 \x01(\x0b\x32\x17.v5api.StreamDescriptor\"\x98\x01\n\x10StreamDescriptor\x12\x0c\n\x04uuid\x18\x01 \x01(\x0c\x12\x12\n\ncollection\x18\x02 \x01(\t\x12 \n\x04tags\x18\x03 \x03(\x0b\x32\x12.v5api.KeyOptValue\x12\'\n\x0b\x61nnotations\x18\x04 \x03(\x0b\x32\x12.v5api.KeyOptValue\x12\x17\n\x0fpropertyVersion\x18\x05 \x01(\x04\"\x82\x01\n\x1aSetStreamAnnotationsParams\x12\x0c\n\x04uuid\x18\x01 \x01(\x0c\x12\x1f\n\x17\x65xpectedPropertyVersion\x18\x02 \x01(\x04\x12#\n\x07\x63hanges\x18\x03 \x03(\x0b\x32\x12.v5api.KeyOptValue\x12\x10\n\x08removals\x18\x04 \x03(\t\";\n\x1cSetStreamAnnotationsResponse\x12\x1b\n\x04stat\x18\x01 \x01(\x0b\x32\r.v5api.Status\"\x8a\x01\n\x13SetStreamTagsParams\x12\x0c\n\x04uuid\x18\x01 \x01(\x0c\x12\x1f\n\x17\x65xpectedPropertyVersion\x18\x02 \x01(\x04\x12 \n\x04tags\x18\x03 \x03(\x0b\x32\x12.v5api.KeyOptValue\x12\x12\n\ncollection\x18\x04 \x01(\t\x12\x0e\n\x06remove\x18\x05 \x03(\t\"4\n\x15SetStreamTagsResponse\x12\x1b\n\x04stat\x18\x01 \x01(\x0b\x32\r.v5api.Status\"{\n\x0c\x43reateParams\x12\x0c\n\x04uuid\x18\x01 \x01(\x0c\x12\x12\n\ncollection\x18\x02 \x01(\t\x12 \n\x04tags\x18\x03 \x03(\x0b\x32\x12.v5api.KeyOptValue\x12\'\n\x0b\x61nnotations\x18\x04 \x03(\x0b\x32\x12.v5api.KeyOptValue\"-\n\x0e\x43reateResponse\x12\x1b\n\x04stat\x18\x01 \x01(\x0b\x32\r.v5api.Status\"@\n\x13MetadataUsageParams\x12\x0e\n\x06prefix\x18\x01 \x01(\t\x12\x19\n\x04role\x18\x64 \x01(\x0b\x32\x0b.v5api.Role\"y\n\x15MetadataUsageResponse\x12\x1b\n\x04stat\x18\x01 \x01(\x0b\x32\r.v5api.Status\x12\x1d\n\x04tags\x18\x02 \x03(\x0b\x32\x0f.v5api.KeyCount\x12$\n\x0b\x61nnotations\x18\x03 \x03(\x0b\x32\x0f.v5api.KeyCount\"&\n\x08KeyCount\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05\x63ount\x18\x02 \x01(\x04\"B\n\x15ListCollectionsParams\x12\x0e\n\x06prefix\x18\x01 \x01(\t\x12\x19\n\x04role\x18\x64 \x01(\x0b\x32\x0b.v5api.Role\"K\n\x17ListCollectionsResponse\x12\x1b\n\x04stat\x18\x01 \x01(\x0b\x32\r.v5api.Status\x12\x13\n\x0b\x63ollections\x18\x02 \x03(\t\"\xab\x01\n\x13LookupStreamsParams\x12\x12\n\ncollection\x18\x01 \x01(\t\x12\x1a\n\x12isCollectionPrefix\x18\x02 \x01(\x08\x12 \n\x04tags\x18\x03 \x03(\x0b\x32\x12.v5api.KeyOptValue\x12\'\n\x0b\x61nnotations\x18\x04 \x03(\x0b\x32\x12.v5api.KeyOptValue\x12\x19\n\x04role\x18\x64 \x01(\x0b\x32\x0b.v5api.Role\"^\n\x15LookupStreamsResponse\x12\x1b\n\x04stat\x18\x01 \x01(\x0b\x32\r.v5api.Status\x12(\n\x07results\x18\x02 \x03(\x0b\x32\x17.v5api.StreamDescriptor\"S\n\rNearestParams\x12\x0c\n\x04uuid\x18\x01 \x01(\x0c\x12\x0c\n\x04time\x18\x02 \x01(\x10\x12\x14\n\x0cversionMajor\x18\x03 \x01(\x04\x12\x10\n\x08\x62\x61\x63kward\x18\x04 \x01(\x08\"z\n\x0fNearestResponse\x12\x1b\n\x04stat\x18\x01 \x01(\x0b\x32\r.v5api.Status\x12\x14\n\x0cversionMajor\x18\x02 \x01(\x04\x12\x14\n\x0cversionMinor\x18\x03 \x01(\x04\x12\x1e\n\x05value\x18\x04 \x01(\x0b\x32\x0f.v5api.RawPoint\"U\n\rChangesParams\x12\x0c\n\x04uuid\x18\x01 \x01(\x0c\x12\x11\n\tfromMajor\x18\x02 \x01(\x04\x12\x0f\n\x07toMajor\x18\x03 \x01(\x04\x12\x12\n\nresolution\x18\x04 \x01(\r\"\x7f\n\x0f\x43hangesResponse\x12\x1b\n\x04stat\x18\x01 \x01(\x0b\x32\r.v5api.Status\x12\x14\n\x0cversionMajor\x18\x02 \x01(\x04\x12\x14\n\x0cversionMinor\x18\x03 \x01(\x04\x12#\n\x06ranges\x18\x04 \x03(\x0b\x32\x13.v5api.ChangedRange\"#\n\tRoundSpec\x12\x0e\n\x04\x62its\x18\x02 \x01(\x05H\x00\x42\x06\n\x04spec\"\x99\x01\n\x0cInsertParams\x12\x0c\n\x04uuid\x18\x01 \x01(\x0c\x12\x0c\n\x04sync\x18\x02 \x01(\x08\x12(\n\x0cmerge_policy\x18\x04 \x01(\x0e\x32\x12.v5api.MergePolicy\x12\"\n\x08rounding\x18\x05 \x01(\x0b\x32\x10.v5api.RoundSpec\x12\x1f\n\x06values\x18\x03 \x03(\x0b\x32\x0f.v5api.RawPoint\"\x91\x01\n\x11\x41rrowInsertParams\x12\x0c\n\x04uuid\x18\x01 \x01(\x0c\x12\x0c\n\x04sync\x18\x02 \x01(\x08\x12(\n\x0cmerge_policy\x18\x03 \x01(\x0e\x32\x12.v5api.MergePolicy\x12\"\n\x08rounding\x18\x04 \x01(\x0b\x32\x10.v5api.RoundSpec\x12\x12\n\narrowBytes\x18\x05 \x01(\x0c\"Y\n\x0eInsertResponse\x12\x1b\n\x04stat\x18\x01 \x01(\x0b\x32\r.v5api.Status\x12\x14\n\x0cversionMajor\x18\x02 \x01(\x04\x12\x14\n\x0cversionMinor\x18\x03 \x01(\x04\"8\n\x0c\x44\x65leteParams\x12\x0c\n\x04uuid\x18\x01 \x01(\x0c\x12\r\n\x05start\x18\x02 \x01(\x10\x12\x0b\n\x03\x65nd\x18\x03 \x01(\x10\"Y\n\x0e\x44\x65leteResponse\x12\x1b\n\x04stat\x18\x01 \x01(\x0b\x32\r.v5api.Status\x12\x14\n\x0cversionMajor\x18\x02 \x01(\x04\x12\x14\n\x0cversionMinor\x18\x03 \x01(\x04\"\x0c\n\nInfoParams\"\xa2\x01\n\x0cInfoResponse\x12\x1b\n\x04stat\x18\x01 \x01(\x0b\x32\r.v5api.Status\x12\x19\n\x04mash\x18\x02 \x01(\x0b\x32\x0b.v5api.Mash\x12\x14\n\x0cmajorVersion\x18\x03 \x01(\r\x12\x14\n\x0cminorVersion\x18\x04 \x01(\r\x12\r\n\x05\x62uild\x18\x05 \x01(\t\x12\x1f\n\x05proxy\x18\x06 \x01(\x0b\x32\x10.v5api.ProxyInfo\"#\n\tProxyInfo\x12\x16\n\x0eproxyEndpoints\x18\x01 \x03(\t\"1\n\x11\x46\x61ultInjectParams\x12\x0c\n\x04type\x18\x01 \x01(\x04\x12\x0e\n\x06params\x18\x02 \x01(\x0c\">\n\x13\x46\x61ultInjectResponse\x12\x1b\n\x04stat\x18\x01 \x01(\x0b\x32\r.v5api.Status\x12\n\n\x02rv\x18\x02 \x01(\x0c\"\x1b\n\x0b\x46lushParams\x12\x0c\n\x04uuid\x18\x01 \x01(\x0c\"X\n\rFlushResponse\x12\x1b\n\x04stat\x18\x01 \x01(\x0b\x32\r.v5api.Status\x12\x14\n\x0cversionMajor\x18\x02 \x01(\x04\x12\x14\n\x0cversionMinor\x18\x03 \x01(\x04\" \n\x10ObliterateParams\x12\x0c\n\x04uuid\x18\x01 \x01(\x0c\"1\n\x12ObliterateResponse\x12\x1b\n\x04stat\x18\x01 \x01(\x0b\x32\r.v5api.Status\"\'\n\x08RawPoint\x12\x0c\n\x04time\x18\x01 \x01(\x10\x12\r\n\x05value\x18\x02 \x01(\x01\"`\n\tStatPoint\x12\x0c\n\x04time\x18\x01 \x01(\x10\x12\x0b\n\x03min\x18\x02 \x01(\x01\x12\x0c\n\x04mean\x18\x03 \x01(\x01\x12\x0b\n\x03max\x18\x04 \x01(\x01\x12\r\n\x05\x63ount\x18\x05 \x01(\x06\x12\x0e\n\x06stddev\x18\x06 \x01(\x01\"*\n\x0c\x43hangedRange\x12\r\n\x05start\x18\x01 \x01(\x10\x12\x0b\n\x03\x65nd\x18\x02 \x01(\x10\">\n\x06Status\x12\x0c\n\x04\x63ode\x18\x01 \x01(\r\x12\x0b\n\x03msg\x18\x02 \x01(\t\x12\x19\n\x04mash\x18\x03 \x01(\x0b\x32\x0b.v5api.Mash\"\x98\x01\n\x04Mash\x12\x10\n\x08revision\x18\x01 \x01(\x03\x12\x0e\n\x06leader\x18\x02 \x01(\t\x12\x16\n\x0eleaderRevision\x18\x03 \x01(\x03\x12\x13\n\x0btotalWeight\x18\x04 \x01(\x03\x12\x0f\n\x07healthy\x18\x05 \x01(\x08\x12\x10\n\x08unmapped\x18\x06 \x01(\x01\x12\x1e\n\x07members\x18\x07 \x03(\x0b\x32\r.v5api.Member\"\xc3\x01\n\x06Member\x12\x0c\n\x04hash\x18\x01 \x01(\r\x12\x10\n\x08nodename\x18\x02 \x01(\t\x12\n\n\x02up\x18\x03 \x01(\x08\x12\n\n\x02in\x18\x04 \x01(\x08\x12\x0f\n\x07\x65nabled\x18\x05 \x01(\x08\x12\r\n\x05start\x18\x06 \x01(\x03\x12\x0b\n\x03\x65nd\x18\x07 \x01(\x03\x12\x0e\n\x06weight\x18\x08 \x01(\x03\x12\x16\n\x0ereadPreference\x18\t \x01(\x01\x12\x15\n\rhttpEndpoints\x18\n \x01(\t\x12\x15\n\rgrpcEndpoints\x18\x0b \x01(\t\"8\n\x0bKeyOptValue\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\x1c\n\x03val\x18\x02 \x01(\x0b\x32\x0f.v5api.OptValue\"\x19\n\x08OptValue\x12\r\n\x05value\x18\x01 \x01(\t\"&\n\x08KeyValue\x12\x0b\n\x03key\x18\x01 \x01(\t\x12\r\n\x05value\x18\x02 \x01(\t\"?\n\x0fStreamCSVConfig\x12\x0f\n\x07version\x18\x01 \x01(\x04\x12\r\n\x05label\x18\x02 \x01(\t\x12\x0c\n\x04uuid\x18\x03 \x01(\x0c\"\x9d\x02\n\x11GenerateCSVParams\x12\x35\n\tqueryType\x18\x01 \x01(\x0e\x32\".v5api.GenerateCSVParams.QueryType\x12\x11\n\tstartTime\x18\x02 \x01(\x03\x12\x0f\n\x07\x65ndTime\x18\x03 \x01(\x03\x12\x12\n\nwindowSize\x18\x04 \x01(\x04\x12\r\n\x05\x64\x65pth\x18\x05 \x01(\r\x12\x17\n\x0fincludeVersions\x18\x06 \x01(\x08\x12\'\n\x07streams\x18\x07 \x03(\x0b\x32\x16.v5api.StreamCSVConfig\"H\n\tQueryType\x12\x19\n\x15\x41LIGNED_WINDOWS_QUERY\x10\x00\x12\x11\n\rWINDOWS_QUERY\x10\x01\x12\r\n\tRAW_QUERY\x10\x02\"Q\n\x13GenerateCSVResponse\x12\x1b\n\x04stat\x18\x01 \x01(\x0b\x32\r.v5api.Status\x12\x10\n\x08isHeader\x18\x02 \x01(\x08\x12\x0b\n\x03row\x18\x03 \x03(\t\"J\n\x0eSQLQueryParams\x12\r\n\x05query\x18\x01 \x01(\t\x12\x0e\n\x06params\x18\x02 \x03(\t\x12\x19\n\x04role\x18\x64 \x01(\x0b\x32\x0b.v5api.Role\"D\n\x10SQLQueryResponse\x12\x1b\n\x04stat\x18\x01 \x01(\x0b\x32\r.v5api.Status\x12\x13\n\x0bSQLQueryRow\x18\x02 \x03(\x0c\"\x14\n\x04Role\x12\x0c\n\x04name\x18\x01 \x01(\t\"\x94\x01\n\x19SetCompactionConfigParams\x12\x0c\n\x04uuid\x18\x01 \x01(\x0c\x12\x18\n\x10\x43ompactedVersion\x18\x02 \x01(\x04\x12>\n\x17reducedResolutionRanges\x18\x03 \x03(\x0b\x32\x1d.v5api.ReducedResolutionRange\x12\x0f\n\x07unused0\x18\x04 \x01(\x04\":\n\x1bSetCompactionConfigResponse\x12\x1b\n\x04stat\x18\x01 \x01(\x0b\x32\r.v5api.Status\")\n\x19GetCompactionConfigParams\x12\x0c\n\x04uuid\x18\x01 \x01(\x0c\"\xc1\x01\n\x1bGetCompactionConfigResponse\x12\x1b\n\x04stat\x18\x01 \x01(\x0b\x32\r.v5api.Status\x12\x1a\n\x12LatestMajorVersion\x18\x02 \x01(\x04\x12\x18\n\x10\x43ompactedVersion\x18\x03 \x01(\x04\x12>\n\x17reducedResolutionRanges\x18\x04 \x03(\x0b\x32\x1d.v5api.ReducedResolutionRange\x12\x0f\n\x07unused0\x18\x05 \x01(\x04\"H\n\x16ReducedResolutionRange\x12\r\n\x05Start\x18\x01 \x01(\x03\x12\x0b\n\x03\x45nd\x18\x02 \x01(\x03\x12\x12\n\nResolution\x18\x03 \x01(\r\"K\n\x12SubscriptionUpdate\x12\'\n\x02op\x18\x01 \x01(\x0e\x32\x1b.v5api.SubscriptionUpdateOp\x12\x0c\n\x04uuid\x18\x02 \x03(\x0c\"Q\n\x10SubscriptionResp\x12\x1b\n\x04stat\x18\x01 \x01(\x0b\x32\r.v5api.Status\x12\x0c\n\x04uuid\x18\x02 \x01(\x0c\x12\x12\n\narrowBytes\x18\x03 \x01(\x0c*<\n\x0bMergePolicy\x12\t\n\x05NEVER\x10\x00\x12\t\n\x05\x45QUAL\x10\x01\x12\n\n\x06RETAIN\x10\x02\x12\x0b\n\x07REPLACE\x10\x03*7\n\x14SubscriptionUpdateOp\x12\r\n\tADD_UUIDS\x10\x00\x12\x10\n\x0cREMOVE_UUIDS\x10\x01\x32\xfd\r\n\x05\x42TrDB\x12?\n\tRawValues\x12\x16.v5api.RawValuesParams\x1a\x18.v5api.RawValuesResponse0\x01\x12N\n\x0e\x41rrowRawValues\x12\x1b.v5api.ArrowRawValuesParams\x1a\x1d.v5api.ArrowRawValuesResponse0\x01\x12T\n\x10\x41rrowMultiValues\x12\x1d.v5api.ArrowMultiValuesParams\x1a\x1f.v5api.ArrowMultiValuesResponse0\x01\x12N\n\x0e\x41lignedWindows\x12\x1b.v5api.AlignedWindowsParams\x1a\x1d.v5api.AlignedWindowsResponse0\x01\x12X\n\x13\x41rrowAlignedWindows\x12\x1b.v5api.AlignedWindowsParams\x1a\".v5api.ArrowAlignedWindowsResponse0\x01\x12\x39\n\x07Windows\x12\x14.v5api.WindowsParams\x1a\x16.v5api.WindowsResponse0\x01\x12\x43\n\x0c\x41rrowWindows\x12\x14.v5api.WindowsParams\x1a\x1b.v5api.ArrowWindowsResponse0\x01\x12@\n\nStreamInfo\x12\x17.v5api.StreamInfoParams\x1a\x19.v5api.StreamInfoResponse\x12^\n\x14SetStreamAnnotations\x12!.v5api.SetStreamAnnotationsParams\x1a#.v5api.SetStreamAnnotationsResponse\x12I\n\rSetStreamTags\x12\x1a.v5api.SetStreamTagsParams\x1a\x1c.v5api.SetStreamTagsResponse\x12\x34\n\x06\x43reate\x12\x13.v5api.CreateParams\x1a\x15.v5api.CreateResponse\x12Q\n\x0fListCollections\x12\x1c.v5api.ListCollectionsParams\x1a\x1e.v5api.ListCollectionsResponse0\x01\x12K\n\rLookupStreams\x12\x1a.v5api.LookupStreamsParams\x1a\x1c.v5api.LookupStreamsResponse0\x01\x12\x37\n\x07Nearest\x12\x14.v5api.NearestParams\x1a\x16.v5api.NearestResponse\x12\x39\n\x07\x43hanges\x12\x14.v5api.ChangesParams\x1a\x16.v5api.ChangesResponse0\x01\x12\x34\n\x06Insert\x12\x13.v5api.InsertParams\x1a\x15.v5api.InsertResponse\x12>\n\x0b\x41rrowInsert\x12\x18.v5api.ArrowInsertParams\x1a\x15.v5api.InsertResponse\x12\x34\n\x06\x44\x65lete\x12\x13.v5api.DeleteParams\x1a\x15.v5api.DeleteResponse\x12.\n\x04Info\x12\x11.v5api.InfoParams\x1a\x13.v5api.InfoResponse\x12\x43\n\x0b\x46\x61ultInject\x12\x18.v5api.FaultInjectParams\x1a\x1a.v5api.FaultInjectResponse\x12\x31\n\x05\x46lush\x12\x12.v5api.FlushParams\x1a\x14.v5api.FlushResponse\x12@\n\nObliterate\x12\x17.v5api.ObliterateParams\x1a\x19.v5api.ObliterateResponse\x12L\n\x10GetMetadataUsage\x12\x1a.v5api.MetadataUsageParams\x1a\x1c.v5api.MetadataUsageResponse\x12\x45\n\x0bGenerateCSV\x12\x18.v5api.GenerateCSVParams\x1a\x1a.v5api.GenerateCSVResponse0\x01\x12<\n\x08SQLQuery\x12\x15.v5api.SQLQueryParams\x1a\x17.v5api.SQLQueryResponse0\x01\x12\x43\n\tSubscribe\x12\x19.v5api.SubscriptionUpdate\x1a\x17.v5api.SubscriptionResp(\x01\x30\x01\x62\x06proto3')

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(DESCRIPTOR, 'btrdb_pb2', _globals)
if _descriptor._USE_C_DESCRIPTORS == False:
  DESCRIPTOR._options = None
  _globals['_MERGEPOLICY']._serialized_start=6600
  _globals['_MERGEPOLICY']._serialized_end=6660
  _globals['_SUBSCRIPTIONUPDATEOP']._serialized_start=6662
  _globals['_SUBSCRIPTIONUPDATEOP']._serialized_end=6717
  _globals['_RAWVALUESPARAMS']._serialized_start=22
  _globals['_RAWVALUESPARAMS']._serialized_end=103
  _globals['_RAWVALUESRESPONSE']._serialized_start=105
  _globals['_RAWVALUESRESPONSE']._serialized_end=230
  _globals['_ARROWRAWVALUESPARAMS']._serialized_start=232
  _globals['_ARROWRAWVALUESPARAMS']._serialized_end=341
  _globals['_ARROWRAWVALUESRESPONSE']._serialized_start=343
  _globals['_ARROWRAWVALUESRESPONSE']._serialized_end=460
  _globals['_ARROWMULTIVALUESPARAMS']._serialized_start=463
  _globals['_ARROWMULTIVALUESPARAMS']._serialized_end=596
  _globals['_ARROWMULTIVALUESRESPONSE']._serialized_start=598
  _globals['_ARROWMULTIVALUESRESPONSE']._serialized_end=673
  _globals['_RAWPOINTVEC']._serialized_start=675
  _globals['_RAWPOINTVEC']._serialized_end=717
  _globals['_ALIGNEDWINDOWSPARAMS']._serialized_start=719
  _globals['_ALIGNEDWINDOWSPARAMS']._serialized_end=825
  _globals['_ALIGNEDWINDOWSRESPONSE']._serialized_start=828
  _globals['_ALIGNEDWINDOWSRESPONSE']._serialized_end=959
  _globals['_ARROWALIGNEDWINDOWSRESPONSE']._serialized_start=961
  _globals['_ARROWALIGNEDWINDOWSRESPONSE']._serialized_end=1083
  _globals['_WINDOWSPARAMS']._serialized_start=1085
  _globals['_WINDOWSPARAMS']._serialized_end=1194
  _globals['_WINDOWSRESPONSE']._serialized_start=1196
  _globals['_WINDOWSRESPONSE']._serialized_end=1320
  _globals['_ARROWWINDOWSRESPONSE']._serialized_start=1322
  _globals['_ARROWWINDOWSRESPONSE']._serialized_end=1437
  _globals['_STREAMINFOPARAMS']._serialized_start=1439
  _globals['_STREAMINFOPARAMS']._serialized_end=1543
  _globals['_STREAMINFORESPONSE']._serialized_start=1546
  _globals['_STREAMINFORESPONSE']._serialized_end=1684
  _globals['_STREAMDESCRIPTOR']._serialized_start=1687
  _globals['_STREAMDESCRIPTOR']._serialized_end=1839
  _globals['_SETSTREAMANNOTATIONSPARAMS']._serialized_start=1842
  _globals['_SETSTREAMANNOTATIONSPARAMS']._serialized_end=1972
  _globals['_SETSTREAMANNOTATIONSRESPONSE']._serialized_start=1974
  _globals['_SETSTREAMANNOTATIONSRESPONSE']._serialized_end=2033
  _globals['_SETSTREAMTAGSPARAMS']._serialized_start=2036
  _globals['_SETSTREAMTAGSPARAMS']._serialized_end=2174
  _globals['_SETSTREAMTAGSRESPONSE']._serialized_start=2176
  _globals['_SETSTREAMTAGSRESPONSE']._serialized_end=2228
  _globals['_CREATEPARAMS']._serialized_start=2230
  _globals['_CREATEPARAMS']._serialized_end=2353
  _globals['_CREATERESPONSE']._serialized_start=2355
  _globals['_CREATERESPONSE']._serialized_end=2400
  _globals['_METADATAUSAGEPARAMS']._serialized_start=2402
  _globals['_METADATAUSAGEPARAMS']._serialized_end=2466
  _globals['_METADATAUSAGERESPONSE']._serialized_start=2468
  _globals['_METADATAUSAGERESPONSE']._serialized_end=2589
  _globals['_KEYCOUNT']._serialized_start=2591
  _globals['_KEYCOUNT']._serialized_end=2629
  _globals['_LISTCOLLECTIONSPARAMS']._serialized_start=2631
  _globals['_LISTCOLLECTIONSPARAMS']._serialized_end=2697
  _globals['_LISTCOLLECTIONSRESPONSE']._serialized_start=2699
  _globals['_LISTCOLLECTIONSRESPONSE']._serialized_end=2774
  _globals['_LOOKUPSTREAMSPARAMS']._serialized_start=2777
  _globals['_LOOKUPSTREAMSPARAMS']._serialized_end=2948
  _globals['_LOOKUPSTREAMSRESPONSE']._serialized_start=2950
  _globals['_LOOKUPSTREAMSRESPONSE']._serialized_end=3044
  _globals['_NEARESTPARAMS']._serialized_start=3046
  _globals['_NEARESTPARAMS']._serialized_end=3129
  _globals['_NEARESTRESPONSE']._serialized_start=3131
  _globals['_NEARESTRESPONSE']._serialized_end=3253
  _globals['_CHANGESPARAMS']._serialized_start=3255
  _globals['_CHANGESPARAMS']._serialized_end=3340
  _globals['_CHANGESRESPONSE']._serialized_start=3342
  _globals['_CHANGESRESPONSE']._serialized_end=3469
  _globals['_ROUNDSPEC']._serialized_start=3471
  _globals['_ROUNDSPEC']._serialized_end=3506
  _globals['_INSERTPARAMS']._serialized_start=3509
  _globals['_INSERTPARAMS']._serialized_end=3662
  _globals['_ARROWINSERTPARAMS']._serialized_start=3665
  _globals['_ARROWINSERTPARAMS']._serialized_end=3810
  _globals['_INSERTRESPONSE']._serialized_start=3812
  _globals['_INSERTRESPONSE']._serialized_end=3901
  _globals['_DELETEPARAMS']._serialized_start=3903
  _globals['_DELETEPARAMS']._serialized_end=3959
  _globals['_DELETERESPONSE']._serialized_start=3961
  _globals['_DELETERESPONSE']._serialized_end=4050
  _globals['_INFOPARAMS']._serialized_start=4052
  _globals['_INFOPARAMS']._serialized_end=4064
  _globals['_INFORESPONSE']._serialized_start=4067
  _globals['_INFORESPONSE']._serialized_end=4229
  _globals['_PROXYINFO']._serialized_start=4231
  _globals['_PROXYINFO']._serialized_end=4266
  _globals['_FAULTINJECTPARAMS']._serialized_start=4268
  _globals['_FAULTINJECTPARAMS']._serialized_end=4317
  _globals['_FAULTINJECTRESPONSE']._serialized_start=4319
  _globals['_FAULTINJECTRESPONSE']._serialized_end=4381
  _globals['_FLUSHPARAMS']._serialized_start=4383
  _globals['_FLUSHPARAMS']._serialized_end=4410
  _globals['_FLUSHRESPONSE']._serialized_start=4412
  _globals['_FLUSHRESPONSE']._serialized_end=4500
  _globals['_OBLITERATEPARAMS']._serialized_start=4502
  _globals['_OBLITERATEPARAMS']._serialized_end=4534
  _globals['_OBLITERATERESPONSE']._serialized_start=4536
  _globals['_OBLITERATERESPONSE']._serialized_end=4585
  _globals['_RAWPOINT']._serialized_start=4587
  _globals['_RAWPOINT']._serialized_end=4626
  _globals['_STATPOINT']._serialized_start=4628
  _globals['_STATPOINT']._serialized_end=4724
  _globals['_CHANGEDRANGE']._serialized_start=4726
  _globals['_CHANGEDRANGE']._serialized_end=4768
  _globals['_STATUS']._serialized_start=4770
  _globals['_STATUS']._serialized_end=4832
  _globals['_MASH']._serialized_start=4835
  _globals['_MASH']._serialized_end=4987
  _globals['_MEMBER']._serialized_start=4990
  _globals['_MEMBER']._serialized_end=5185
  _globals['_KEYOPTVALUE']._serialized_start=5187
  _globals['_KEYOPTVALUE']._serialized_end=5243
  _globals['_OPTVALUE']._serialized_start=5245
  _globals['_OPTVALUE']._serialized_end=5270
  _globals['_KEYVALUE']._serialized_start=5272
  _globals['_KEYVALUE']._serialized_end=5310
  _globals['_STREAMCSVCONFIG']._serialized_start=5312
  _globals['_STREAMCSVCONFIG']._serialized_end=5375
  _globals['_GENERATECSVPARAMS']._serialized_start=5378
  _globals['_GENERATECSVPARAMS']._serialized_end=5663
  _globals['_GENERATECSVPARAMS_QUERYTYPE']._serialized_start=5591
  _globals['_GENERATECSVPARAMS_QUERYTYPE']._serialized_end=5663
  _globals['_GENERATECSVRESPONSE']._serialized_start=5665
  _globals['_GENERATECSVRESPONSE']._serialized_end=5746
  _globals['_SQLQUERYPARAMS']._serialized_start=5748
  _globals['_SQLQUERYPARAMS']._serialized_end=5822
  _globals['_SQLQUERYRESPONSE']._serialized_start=5824
  _globals['_SQLQUERYRESPONSE']._serialized_end=5892
  _globals['_ROLE']._serialized_start=5894
  _globals['_ROLE']._serialized_end=5914
  _globals['_SETCOMPACTIONCONFIGPARAMS']._serialized_start=5917
  _globals['_SETCOMPACTIONCONFIGPARAMS']._serialized_end=6065
  _globals['_SETCOMPACTIONCONFIGRESPONSE']._serialized_start=6067
  _globals['_SETCOMPACTIONCONFIGRESPONSE']._serialized_end=6125
  _globals['_GETCOMPACTIONCONFIGPARAMS']._serialized_start=6127
  _globals['_GETCOMPACTIONCONFIGPARAMS']._serialized_end=6168
  _globals['_GETCOMPACTIONCONFIGRESPONSE']._serialized_start=6171
  _globals['_GETCOMPACTIONCONFIGRESPONSE']._serialized_end=6364
  _globals['_REDUCEDRESOLUTIONRANGE']._serialized_start=6366
  _globals['_REDUCEDRESOLUTIONRANGE']._serialized_end=6438
  _globals['_SUBSCRIPTIONUPDATE']._serialized_start=6440
  _globals['_SUBSCRIPTIONUPDATE']._serialized_end=6515
  _globals['_SUBSCRIPTIONRESP']._serialized_start=6517
  _globals['_SUBSCRIPTIONRESP']._serialized_end=6598
  _globals['_BTRDB']._serialized_start=6720
  _globals['_BTRDB']._serialized_end=8509
# @@protoc_insertion_point(module_scope)
